package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.collections.list.JArrayList;
import com.xy.collections.list.JList;
import com.xy.collections.map.JMap;
import com.xy.dto.DeviceQualityDto;
import com.xy.entity.DeviceInfo;
import com.xy.entity.DeviceQuality;
import com.xy.mapper.DeviceInfoMapper;
import com.xy.mapper.DeviceQualityMapper;
import com.xy.utils.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import static com.xy.utils.PlusBeans.*;

/**
 * <p>
 * 设备-质检 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-01-05
 */
@Service
@AllArgsConstructor
@Api(tags = "设备-质检")
public class DeviceQualityServiceImpl extends ServiceImpl<DeviceQualityMapper, DeviceQuality> implements DeviceQualityService {

    private DeviceInfoMapper deviceInfoMapper;

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<DeviceQualityDto.Vo>> page(@RequestBody DeviceQualityDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<DeviceQuality> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, DeviceQuality.class)
                .ge(DeviceQuality::getUpdateTime, page.getBeginUpdateTime())
                .le(DeviceQuality::getUpdateTime, page.getEndUpdateTime())
                .build()
                .orderByDesc(!Emptys.check(pageBean.getOrders()), DeviceQuality::getCreateTime);
        IPage<DeviceQuality> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        return R.ok(toPageBean(DeviceQualityDto.Vo.class, iPage));
    }

    @PostMapping("update")
    @ApiOperation("修改")
    public R update(@RequestBody List<DeviceQualityDto.Vo> vos) {
        List<DeviceQuality> deviceQualities = copy(DeviceQuality.class, vos);
        LocalDateTime now = LocalDateTime.now();
        for (DeviceQuality deviceQuality : deviceQualities) {
            if (deviceQuality.getQualityState() != null || deviceQuality.getFactoryState() != null) {
                deviceQuality.setQualityAuditUserId(AuthorizeUtils.getLoginId(Long.class));
            }
            deviceQuality.setUpdateTime(now);
        }
        updateBatchById(deviceQualities);
        return R.ok();
    }

    @PostMapping("save")
    @ApiOperation("添加")
    @Transactional(rollbackFor = Exception.class)
    public R save(@RequestBody List<DeviceQualityDto.Vo> vos) {
        List<DeviceQualityDto.Vo> updates = new ArrayList<>();
        List<DeviceQuality> saves = new ArrayList<>();
        JList<Long> deviceIds = new JArrayList<>(vos).getProperty(DeviceQualityDto.Vo::getDeviceId);
        //获取设备信息
        List<DeviceInfo> deviceInfos = deviceInfoMapper.selectList(new LambdaQueryWrapper<DeviceInfo>().in(DeviceInfo::getDeviceId, deviceIds));
        JMap<Long, DeviceInfo> deviceInfoJMap = new JArrayList<>(deviceInfos).toMap(DeviceInfo::getDeviceId).cover();
        //获取已有质检信息
        List<DeviceQuality> deviceQualities = list(new LambdaQueryWrapper<DeviceQuality>().in(DeviceQuality::getDeviceId, deviceIds));
        JMap<Long, List<DeviceQuality>> deviceQualitieJMap = new JArrayList<>(deviceQualities).toMap(DeviceQuality::getDeviceId).group();
        //封装修改和添加对象
        for (DeviceQualityDto.Vo save : vos) {
            DeviceQuality deviceQuality = new DeviceQuality().setDeviceId(save.getDeviceId());
            DeviceInfo deviceInfo = deviceInfoJMap.get(deviceQuality.getDeviceId());
            if (deviceInfo == null) {
                return R.fail(save.getDeviceId() + "设备不存在");
            }
            deviceQuality.setDeviceType(deviceInfo.getDeviceType())
                    .setCreateTime(LocalDateTime.now());
            List<DeviceQuality> list = deviceQualitieJMap.get(deviceQuality.getDeviceId());
            if (Emptys.check(list)) {
                JList<Integer> qualityIds = new JArrayList<>(list).getProperty(DeviceQuality::getQualityId);
                List<DeviceQualityDto.Vo> vos2 = new ArrayList<>(qualityIds.size());
                qualityIds.forEach(qualityId -> vos2.add(new DeviceQualityDto.Vo().setQualityId(qualityId).setShowStatus(false)));
                updates.addAll(vos2);
            }
            saves.add(deviceQuality);
        }
        //修改
        if (Emptys.check(updates)) {
            update(updates);
        }
        //添加
        saveBatch(saves);
        return R.ok();
    }

    @PostMapping("list")
    @ApiOperation("集合查询")
    public R<List<DeviceQualityDto.Vo>> list(@RequestBody DeviceQualityDto.SelectList selectList) {
        LambdaQueryWrapper<DeviceQuality> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList, DeviceQuality.class)
                .in(DeviceQuality::getQualityId, selectList.getQualityIds())
                .in(DeviceQuality::getDeviceId, selectList.getDeviceIds())
                .build();
        List<DeviceQuality> list = list(lambdaQueryWrapper);
        return R.ok(copy(DeviceQualityDto.Vo.class, list));
    }

    @PostMapping("getQrCode")
    @ApiOperation("获取二维码")
    public R<String> getQrCode(@RequestBody DeviceQualityDto.Qrcode qrcode) {
        String deviceId = qrcode.getDeviceId().toString();
        String base64 = QRCodeUtils.create(deviceId, 400, 400, "back1.png", deviceId).base64();
        return R.ok(base64);
    }

    @PostMapping("exportQrCode")
    @ApiOperation("导出二维码")
    public void exportQrCode(@RequestBody DeviceQualityDto.Qrcode qrcode) {
        String deviceId = qrcode.getDeviceId().toString();
        QRCodeUtils.create(deviceId, 400, 400, "back1.png", deviceId).download(deviceId);
    }

}
